-----------------------------------------------------------------------------------------------
-- SimpleTrack by James Davidson (Feel free to do whatever you want with this code)
-- Requires TrackMaster
-----------------------------------------------------------------------------------------------

require "Window"
 
-----------------------------------------------------------------------------------------------
-- SimpleTrack Module Definition
-----------------------------------------------------------------------------------------------
local SimpleTrack = {} 

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimpleTrack:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    self.target = ""
	self.enabled = true
	self.units = {}
	
    return o
end

function SimpleTrack:Init()
    Apollo.RegisterAddon(self)
end

-----------------------------------------------------------------------------------------------
-- Load/Save/Restore Functions
-----------------------------------------------------------------------------------------------
function SimpleTrack:OnLoad()
	-- Register slash commands
	Apollo.RegisterSlashCommand("st", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("simpletrack", "OnSlashCommand", self)
	
	-- Register event handlers to match when units are created and destroyed
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
end

function SimpleTrack:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	local saveData = { }
	saveData.Enabled = self.enabled
	saveData.Target = self.target
	return saveData
end

function SimpleTrack:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end
	
	if tData.Enabled then
		self:EnableST()
	else
		self:DisableST()
	end
	
	if tData.Target then
		self.target = tData.Target
	end
end

-----------------------------------------------------------------------------------------------
-- SimpleTrack GetAsyncLoadStatus
-----------------------------------------------------------------------------------------------
function SimpleTrack:GetAsyncLoadStatus()
	-- check for external dependencies here
	if g_AddonsLoaded == nil then
		g_AddonsLoaded = {}
	end
	if not g_AddonsLoaded["TrackMaster"] then
		return Apollo.AddonLoadStatus.Loading
	end

	self.timer = ApolloTimer.Create(1.0, true, "OnTimer", self)
		
	g_AddonsLoaded["SimpleTrack"] = true
	
	return Apollo.AddonLoadStatus.Loaded
end

-----------------------------------------------------------------------------------------------
-- Timer
-----------------------------------------------------------------------------------------------
function SimpleTrack:OnTimer()
	if self.enabled and GameLib.GetPlayerUnit() then
		local playerPos = GameLib.GetPlayerUnit():GetPosition()
		local closestUnit = nil
		local closestUnitDist = 0xffffff
		for _, unit in pairs(self.units) do
			local pos = unit:GetPosition()
			local dist = self:CalculateDistance(Vector3.New(playerPos.x - pos.x, playerPos.y - pos.y, playerPos.z - pos.z))
	
			if dist < closestUnitDist and string.lower(unit:GetName()) == string.lower(self.target) then
				closestUnit = unit
				closestUnitDist = dist
			end
		end
		
		if closestUnit ~= nil then
			Apollo.GetAddon("TrackMaster"):SetTarget(closestUnit, 5)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- SimpleTrack functions
-----------------------------------------------------------------------------------------------
function SimpleTrack:CalculateDistance(vector)
	return math.sqrt(math.pow(vector.x, 2) + math.pow(vector.y, 2) + math.pow(vector.z, 2))
end

function SimpleTrack:OnUnitCreated(unit)
	self.units[unit:GetId()] = unit
end

function SimpleTrack:OnUnitDestroyed(unit)
	if self.units[unit:GetId()] ~= nil then 
		self.units[unit:GetId()] = nil
	end
end

-----------------------------------------------------------------------------------------------
-- Enable / Disable Functions
-----------------------------------------------------------------------------------------------
function SimpleTrack:EnableST()
	if not self.enabled then
		self.enabled = true
		self.timer:Start()
		if self.target ~= "" then
			self:WriteToChat("SimpleTrack enabled. Tracking \"" .. self.target .. "\".")
		else
			self:WriteToChat("SimpleTrack enabled. Not tracking anything. Type /st t X to set target.")
		end
	end
end

function SimpleTrack:DisableST()
	if self.enabled then
		self.enabled = false
		if self.timer ~= nil then
			self.timer:Stop()
		end
		self:WriteToChat("SimpleTrack disabled.")
	end
end

-----------------------------------------------------------------------------------------------
-- Chat Command Handlers (Need to turn this into a GUI!!!!!!)
-----------------------------------------------------------------------------------------------
function SimpleTrack:OnSlashCommand(strCmd, strParam)
	if strParam ~= "" then
		local tParams = {}
		for w in string.gmatch(strParam, "%S*") do
			if w ~= "" then
				if tParams[2] ~= nil then
					tParams[2] = tParams[2] .. " " .. w
				else
					table.insert(tParams, w)
				end
			end
		end
		if string.lower(tParams[1]) == "status" then
			self:ShowStatusInChat()
		elseif string.lower(tParams[1]) == "enable" then
			self:EnableST()
		elseif string.lower(tParams[1]) == "disable" then
			self:DisableST()
		elseif string.lower(tParams[1]) == "clear" then
			self.target = ""
			self:WriteToChat("SimpleTrack target cleared.")
		elseif string.lower(tParams[1]) == "target" then
			if tParams[2] ~= nil and tParams[2] ~= "" then
				self:SetTargetFromChat(tParams[2])
			else
				if self.target ~= "" then
					self:WriteToChat("SimpleTrack tracking \"" .. self.target .. "\".")
				else
					self:WriteToChat("SimpleTrack not tracking anything.")
				end
			end
		elseif string.lower(tParams[1]) == "t" then
			if tParams[2] ~= nil and tParams[2] ~= "" then
				self:SetTargetFromChat(tParams[2])
			else
				if self.target ~= "" then
					self:WriteToChat("SimpleTrack tracking \"" .. self.target .. "\".")
				else
					self:WriteToChat("SimpleTrack not tracking anything.")
				end
			end
		else
			self:ShowHelp()
		end
	else
		self:ShowHelp()
	end
end

function SimpleTrack:ShowHelp()
	self:ShowStatusInChat();
	self:WriteToChat("Available commands:")
	self:WriteToChat("-- \"enable\" enables SimpleTrack.")
	self:WriteToChat("-- \"disable\" disables SimpleTrack.")
	self:WriteToChat("-- \"target X\" set the target name to track.")
	self:WriteToChat("-- \"t X\" same as above, just a shorter version.")
	self:WriteToChat("-- \"status\" only display status information.")
	self:WriteToChat("-- \"clear\" clear target information.")
end

function SimpleTrack:ShowStatusInChat()
	local status = "enabled"
	if self.enabled ~= true then status = "disabled" end
	self:WriteToChat("SimpleTrack is currently " .. status .. ".")
	if self.enabled and self.target ~= "" then
		self:WriteToChat("-- Tracking \"" .. self.target .. "\".")
	end
end

function SimpleTrack:SetTargetFromChat(tgt)
	if tgt ~= nil and tgt ~= "" then
		self.target = tgt
		if self.enabled ~= true then
			self:WriteToChat("SimpleTrack would be tracking \"" .. self.target .. "\" but it's currently disabled. Enable it!")
		else
			self:WriteToChat("SimpleTrack now tracking \"" .. self.target .. "\".")
		end
	end
end

function SimpleTrack:WriteToChat(text)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, text)
end

-----------------------------------------------------------------------------------------------
-- SimpleTrack Instance
-----------------------------------------------------------------------------------------------
local SimpleTrackInst = SimpleTrack:new()
SimpleTrack:Init()